package org.gridschool.unicore.plugins.boltzmann;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.unicore.AJOIdentifier;
import org.unicore.Vsite;
import org.unicore.outcome.AbstractJob_Outcome;

import com.pallas.unicore.client.editor.RemoteTextEditor;
import com.pallas.unicore.client.panels.Applyable;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.threadpool.IObserver;
import com.pallas.unicore.utility.StringTools;

/**
 * Additional outcome panel for steering plugin.
 *
 *@author Ralf Ratering
 *@version    $Id: BoltzmannControlPanel.java,v 1.3 2003/07/12 17:09:43 rrate Exp $
 */
public class BoltzmannControlPanel extends JPanel implements IObserver, Applyable {
    static final long serialVersionUID = 9175578907213854759L;

	private static Logger logger =
			Logger.getLogger("org.gridschool.unicore.plugins.boltzmann");

	private static final String CONTROL_FILE = "control";

	private RemoteTextEditor editor;
	
    /**
     * Constructor
     */
    public BoltzmannControlPanel() {
		buildComponents();
    }
    
  
    private void buildComponents() {
		JPanel topPanel = new JPanel();

		JButton sendButton = new JButton("Send Control File");
		sendButton.addActionListener(new SendListener());
		topPanel.add(sendButton);
		
		editor = new RemoteTextEditor();
						
		setLayout(new BorderLayout());
    	add(topPanel, BorderLayout.NORTH);
		add(new JScrollPane(editor), BorderLayout.CENTER);
    }


    private JobContainer getJobContainer() {
    	return
	    	ResourceManager.getCurrentInstance().getJMCTree().getCurrentJob();
    }

    private BoltzmannContainer getBoltzmannContainer() {
		return (BoltzmannContainer)
			ResourceManager.getCurrentInstance().getJMCTree().getFocussedObject();
    }

	/**
	 * 
	 * The Boltzmann application will store intermediate results in
	 * a file called sample.gif. We want to transfer this file to
	 * the Client while the job is still running, so we have to
	 * wrap it inside a Portfolio, because we will use a
	 * CopyPortfolioToOutcome for the transfer.
     */	
    private void sendControlFile() {
    	logger.info("Sending  control file");
    	JobContainer jobContainer = getJobContainer();
	   	AJOIdentifier ajoId = (AJOIdentifier)jobContainer.getIdentifier();
		Vsite vsite = jobContainer.getVsite();
		String[] filenames = {CONTROL_FILE};
	
		// get input file contents as 2D byte array
		byte[][] contents = new byte[1][];
		String inputString = StringTools.dos2Unix(editor.getText());
		contents[0] = inputString.getBytes();
			
		SendFilesToAlternativeUspace request = 
			new SendFilesToAlternativeUspace(ajoId, filenames, contents, vsite);
		request.addObserver(this);
		request.start();

    }

	public void observableUpdate(Object theObserved, Object changeCode) {
		if (theObserved instanceof SendFilesToAlternativeUspace) {
			logger.info("SendFilesToAlternativeUspace request returned.");
			// SUCCESS! Now get the outcome and implicitly unspool outcome files
			AbstractJob_Outcome outcome = (AbstractJob_Outcome)changeCode;
			logger.info("SendFilesToAlternativeUspace result: " + outcome.getStatus());
		}
	}

	private class SendListener implements ActionListener, Serializable {
		public void actionPerformed(ActionEvent e) {
			sendControlFile();
		}
	}
	/** 
	 * @see com.pallas.unicore.client.panels.Applyable#applyValues()
	 */
	public void applyValues() {
		// do nothing
	}

	/**
	 * @see com.pallas.unicore.client.panels.Applyable#resetValues()
	 */
	public void resetValues() {
		//		do nothing
	}
	
	/**
	 * @see com.pallas.unicore.client.panels.Applyable#updateValues()
	 */
	public void updateValues() {
	}

}
