package org.gridschool.unicore.plugins.boltzmann;

import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.client.util.TaskPlugable;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.GroupContainer;

/**
 *  Main class for boltzmann plugin example
 *
 *@author     Ralf Ratering
 *@version    $Id: BoltzmannPlugin.java,v 1.1 2003/07/09 16:50:35 rrate Exp $
 */
public class BoltzmannPlugin extends TaskPlugable {

    /**
     *  Return a new container for this plugin
     *
     *@param  parentContainer  job that this task belongs to
     *@return                  new initialized instance of plugin container
     */
    public ActionContainer getContainerInstance(GroupContainer parentContainer) {
        // build new plugin container
		BoltzmannContainer container = new BoltzmannContainer(parentContainer);
		container.setName("New_" + getName() + counter);
		
		// count instance
		counter++;

		return container;
    }


    /**
     * Gets the path to the task icon  
     * The super classes will do the icon handling.
     *
     *@return    path to the plugin icon
     */
    public String getIconPath() {
        return "org/gridschool/unicore/plugins/boltzmann/boltzmann.gif";
    }


    /**
     *  Get name for plugin task application
     *
     *@return    The name value
     */
    public String getName() {
        return "Boltzmann";
    }


    /**
     *  Build a new panel that manages the parameters of a plugin container
     *
     *@param  container  ActionContainer that the panel belongs to
     *@return            panel that represents parameters in job preparation area
     */
    public JPAPanel getPanelInstance(ActionContainer container) {
        // return new instance of plugin JPAPanel
        return new BoltzmannJPAPanel(getClient(), (BoltzmannContainer)container);
    }

    /**
     *  Get a short info message for the plugin
     *
     *@return    String with info message
     */
    public String getPluginInfo() {
        return "Grid Summer School Example: The Boltzmann Plugin\n" +
                "(C)2003 Pallas GmbH";
    }


    /**
     *  Called to inititialize the plugin
     */
    public void startPlugin() {
    	// nothing to do here for our example
    }


    /**
     *  Called when terminating
     */
    public void stopPlugin() {
		// nothing to do here for our example
    }

}
