package org.gridschool.unicore.plugins.boltzmann;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.logging.Level;

import org.unicore.AJOIdentifier;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.DeclarePortfolio;
import org.unicore.ajo.IncarnateFiles;
import org.unicore.ajo.MakePortfolio;
import org.unicore.ajo.Portfolio;
import org.unicore.ajo.PutPortfolio;
import org.unicore.ajo.ActionGroup.InvalidDependencyException;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.resources.AlternativeUspace;
import org.unicore.sets.ResourceSet;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;

import com.pallas.unicore.requests.ObservableRequestThread;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Send a list of files to another job's uspace
 *
 *@author     Ralf Ratering
 *@version $Id: SendFilesToAlternativeUspace.java,v 1.2 2003/07/12 16:21:29 rrate Exp $
 */
public class SendFilesToAlternativeUspace extends ObservableRequestThread {

	private AJOIdentifier ajoId;
	private String[] filenames;
	private byte[][] contents;
	private Vsite vsite;

	/**
	* Set parameters of request in constructor
	* @param ajoId 		identifier for job that possesses uspace
	* @param filenames 	array of filenames to transfer
	* @param vsite 		destination vsite
	*/
	public SendFilesToAlternativeUspace(
		AJOIdentifier ajoId,
		String[] filenames,
		byte[][] contents,
		Vsite vsite) {
		this.ajoId = ajoId;
		this.filenames = filenames;
		this.contents = contents;
		this.vsite = vsite;
	}

	/**
	* Start the request
	*/
	public void run() {
		// setup incarnate files task to transfer files to this jobs uspace
		IncarnateFiles incarnateFiles =
			new IncarnateFiles(
				"INCARNATEFILES",
				new ResourceSet(),
				filenames,
				contents,
				true);

		// wrap source files into portfolio
		MakePortfolio makeSourcePortfolio = new MakePortfolio("MakePortfolio");
		makeSourcePortfolio.setFiles(filenames);
		
		AlternativeUspace uspace = new AlternativeUspace();
		uspace.setUspace(ajoId);
		ResourceSet uspaceResourceSet = new ResourceSet(uspace);
	
		// If target is null, NJS creates a Portfolio at run-time, but need one now,
		// for DeclarePortfolio, so use a locally created one
		Portfolio putTarget = new Portfolio("");
	
		PutPortfolio putPortfolio = new PutPortfolio("PutPortfolio");
		putPortfolio.setSource(makeSourcePortfolio.getPortfolio());
		putPortfolio.setResources(uspaceResourceSet);
		putPortfolio.setTarget(putTarget.getId());
		
		DeclarePortfolio declarePortfolio = new DeclarePortfolio("DeclarePortfolio");
		declarePortfolio.setResources(uspaceResourceSet);
		declarePortfolio.setPortfolio(putTarget);
		
		String servicePrefix = ResourceManager.getServicePrefix();
		User user = ResourceManager.getUser(vsite);
		AbstractJob job =
			new AbstractJob(
				servicePrefix
					+ "SendFilesToAlternativeUspace"
					+ ResourceManager.getNextObjectIdentifier(),
				null,
				null,
				vsite,
				user);
		job.add(incarnateFiles);
		job.add(makeSourcePortfolio);
		job.add(putPortfolio);
		job.add(declarePortfolio);
		try {
			job.addDependency(incarnateFiles, makeSourcePortfolio);
			job.addDependency(makeSourcePortfolio, putPortfolio);
			job.addDependency(putPortfolio, declarePortfolio);
		} 
		catch (InvalidDependencyException e) {
			logger.log(Level.SEVERE, "Could not add dependency.", e);
		}

		Reply reply = nonPolling(job, vsite, user);
		AbstractJob_Outcome ajOut = null;

		// now analyze outcome
		if (!(reply instanceof RetrieveOutcomeReply)) {
			logger.warning(
				"Virtual site not available: "
					+ vsite
					+ "\nReason: "
					+ reply.getLastEntry().getComment());
			return;
		} 
		else {
			try {
				RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
				// deserialize the Outcome Object
				byte[] outcomeBytes = roReply.getOutcome();
				Outcome outcome =
					(Outcome) new ObjectInputStream(
						new ByteArrayInputStream(outcomeBytes))
						.readObject();
				ajOut = ConsignForm.convertFrom((AbstractJob_Outcome) outcome);

			} 
			catch (Exception e) {
				logger.log(Level.SEVERE, "Could send files to alternative uspace.", e);
			}
		}
		notifyObservers(this, ajOut);

	}

}
